****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_t5_create_gaps_table.sas  
*
* Created (mm/dd/yyyy): 10/13/2017
* Last modified: 02/05/2019
* Version: 2.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE: Computes distribution of episode gaps, formats table for report. This macro creates 
*		   datasets for tables 12, 13, and 14. All are created at one time.
*
*  Program inputs:                                                                                   
*  	-
* 
*  Program outputs:                                                                                                                                       
*  	-
* 
*  PARAMETERS:        

*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*	1.1		  03/15/18	  AP		Updated label for gap 999 (QRPL-188)
*
*   1.2       07/17/18	  RT        Added Min, Max, Q1 and Q3 (DEV-1405)
*
*   2.0       12/31/18	  AP        Modified for intergration into QRP
*
*   2.1       02/05/19	  AP        Added two level stratifications
*
***************************************************************************************************;

%macro ms_t5_create_gaps_table();

	%put =====> MACRO CALLED: ms_t5_create_gaps_table v2.1;

	data agg_gaps1;
		set agg_gaps;
		*recode -1 gap length (possible when gapnum = 999, and episode ends on end of query/enrollment;
		if gaplength = -1 then gaplength = 0;
	run;

	proc contents data=agg_gaps out=agg_gaps_outnames(keep=name) noprint;
	run;
	proc sql noprint;
		select distinct name into: classvarlist separated by ' '
		from agg_gaps_outnames
		where lowcase(name) ne 'npts' and lowcase(name) ne 'episodes';
	quit;
	%put &classvarlist;

	*Re-aggregate after recoding;
	proc means data=agg_gaps1 noprint nway;
		var episodes;
		class runid &classvarlist. / missing;
		output out=agg_gaps1(drop=_:) sum=;
	run;

	data _gapnumber_heading;
		format gapnum $14.;
		sort = 1;
		gapnum = 'Gap Number';
		output;
	run;

	data _distribution;
		set agg_gaps1;
			*classify gapnum;
			if gapnum ne . then do;
				if gapnum = 1 then gapcat = 1;
				else if 2<= gapnum < 999 then gapcat =2;
				else if gapnum = 999 then gapcat =3 ;
			end;
			else do;
				gapcat = 0;
			end;
	run;

	/*************************************/
	/* Loop through each stratifications */
	/*************************************/

	data _gapstables;
		set &tablefile(where=(table in ('T12', 'T13', 'T14')));
	run;

	*put distinct stratifications into list;
	proc sql noprint;
		select distinct tablesub into: gaps_strat separated by ' '
		from _gapstables;
	quit;

	%do strat = 1 %to %sysfunc(countw(&gaps_strat.));
		%let tablesub = %scan(&gaps_strat., &strat.);

		*reset macro vars;
		%let gaplevel = ;

		data _null_;
 			set _gapstables(where=(tablesub = "&tablesub."));
			if _n_ =1 then do;
				call symputx('gaplevel', levelid1);
			end;
		run;

		%if "&tablesub" = "A" %then %let stratvars = ;
		%if "&tablesub" = "B" %then %let stratvars = sex ;
		%if "&tablesub" = "C" %then %let stratvars = agegroup agegroupnum;
		%if "&tablesub" = "D" %then %let stratvars = race;
		%if "&tablesub" = "E" %then %let stratvars = hispanic;
		%if "&tablesub" = "BC" %then %let stratvars = sex agegroup agegroupnum;
		%if "&tablesub" = "BD" %then %let stratvars = sex race;
		%if "&tablesub" = "BE" %then %let stratvars = sex hispanic;
		%if "&tablesub" = "CD" %then %let stratvars = agegroup agegroupnum race;
		%if "&tablesub" = "CE" %then %let stratvars = agegroup agegroupnum hispanic;

		*compute various gaplength metrics;

			*T12/T13 - by gap number;
			proc means data=_distribution(where=((level = "&gaplevel") and gaplength >=0)) noprint nway;
				var gaplength;
				class runid group gapcat &stratvars.;
				freq episodes;
				output out=_bygap_&tablesub.1(rename=_freq_=n drop=_type_) median=median mean=mean std= std min=min max=max p25=q1 p75=q3;
			run;

			*merge in squared table - freq statement drops rows where episodes=0;
			proc sort data=_distribution(where=(level = "&gaplevel")) nodupkey out=_bygap_&tablesub.2(keep=runid group gapcat &stratvars.);
				by runid group gapcat &stratvars.;
			run;

			data _bygap_&tablesub.;
				merge _bygap_&tablesub.1(in=a) 	
					  _bygap_&tablesub.2(in=b);
				by runid group gapcat &stratvars.;

				if b and not a then do;
					n=0;
				end;
			run;

			*T14 - all gaps;
			proc means data=_distribution(where=((level = "&gaplevel") and gaplength >=0)) noprint nway;
				var gaplength;
				class runid group &stratvars.;
				freq episodes;
				output out=_allgaps_&tablesub.1(rename=_freq_=n drop=_type_) median=median mean=mean std= std min=min max=max p25=q1 p75=q3;
			run;

			*merge in squared table - freq statement drops rows where episodes=0;
			proc sort data=_distribution(where=(level = "&gaplevel")) nodupkey out=_allgaps_&tablesub.2(keep=runid group gapcat &stratvars.);
				by runid group &stratvars.;
			run;

			data _allgaps_&tablesub.;
				merge _allgaps_&tablesub.1(in=a) 	
					  _allgaps_&tablesub.2(in=b);
				by runid group &stratvars.;

				if b and not a then do;
					n=0;
				end;
			run;

			proc datasets nowarn noprint lib=work;
				delete _bygap_&tablesub.1 _bygap_&tablesub.2 _allgaps_&tablesub.1 _allgaps_&tablesub.2;
			quit;

			/* Compute overall if A table not requested */
			%isdata(dataset=_bygap_A);
			%if %eval(&nobs.<1) %then %do;
				*T12/T13 - by gap number;
				proc means data=_distribution(where=((level = "&gaplevel") and gaplength >=0)) noprint nway;
					var gaplength;
					class runid group gapcat;
					freq episodes;
					output out=_bygap_A1(rename=_freq_=n drop=_type_) median=median mean=mean std= std min=min max=max p25=q1 p75=q3;
				run;

				*merge in squared table - freq statement drops rows where episodes=0;
				proc sort data=_distribution(where=(level = "&gaplevel")) nodupkey out=_bygap_A2(keep=runid group gapcat);
					by runid group gapcat;
				run;

				data _bygap_A;
					merge _bygap_A1(in=a) 	
						  _bygap_A2(in=b);
					by runid group gapcat;

					if b and not a then do;
						n=0;
					end;
				run;

				*T14 - all gaps;
				proc means data=_distribution(where=((level = "&gaplevel") and gaplength >=0)) noprint nway;
					var gaplength;
					class runid group;
					freq episodes;
					output out=_allgaps_A1(rename=_freq_=n drop=_type_) median=median mean=mean std= std min=min max=max p25=q1 p75=q3;
				run;

				*merge in squared table - freq statement drops rows where episodes=0;
				proc sort data=_distribution(where=(level = "&gaplevel")) nodupkey out=_allgaps_A2(keep=runid group gapcat);
					by runid group;
				run;

				data _allgaps_A;
					merge _allgaps_A1(in=a) 	
						  _allgaps_A2(in=b);
					by runid group;

					if b and not a then do;
						n=0;
					end;
				run;

				proc datasets nowarn noprint lib=work;
					delete _bygap_A1 _bygap_A2 _allgaps_A1 _allgaps_A2;
				quit;
			%end;

	    data t12&tablesub.(rename=(n_tmp=n mean_tmp=mean std_tmp=std median_tmp=median
                                   min_tmp=min max_tmp=max q1_tmp=p25 q3_tmp=p75) )
			 t13&tablesub.(rename=(n_tmp=n mean_tmp=mean std_tmp=std median_tmp=median
                                   min_tmp=min max_tmp=max q1_tmp=p25 q3_tmp=p75) )
			 t14&tablesub.(rename=(n_tmp=n mean_tmp=mean std_tmp=std median_tmp=median
                                   min_tmp=min max_tmp=max q1_tmp=p25 q3_tmp=p75) );
			 	set _bygap_&tablesub.(in=a)
					_allgaps_&tablesub.(in=b)
					%if "&tablesub" ne "A" %then %do;
						_bygap_A(in=c)
						_allgaps_A(in=d)
					%end;
					;

			format n_tmp $12. mean_tmp std_tmp median_tmp min_tmp max_tmp q1_tmp q3_tmp $10. gaplabel $50.;
				
			if missing(n)=0 then n_tmp = strip(put(n, comma12.0));
			else n_tmp=strip(put(0,6.0));

			if n ne 0 then do;

				if missing(mean)=0 then mean_tmp = strip(put(mean,8.2));
				else mean_tmp='-';

				if missing(median)=0 then median_tmp = strip(put(median,8.0));
				else median_tmp=strip(put(0,6.1));

				if missing(min)=0 then min_tmp  = strip(put(min,8.0));
				else min_tmp=strip(put(0,6.1));

				if missing(max)=0 then max_tmp = strip(put(max,comma10.0));
				else max_tmp=strip(put(0,6.1));

				if missing(q1)=0 then q1_tmp = strip(put(q1,8.0));
				else q1_tmp=strip(put(0,6.1));

				if missing(q3)=0 then q3_tmp = strip(put(q3,8.0));
				else q3_tmp=strip(put(0,6.1));
			
				if missing(std)=0 then std_tmp =strip(put(std,8.2));
				else std_tmp='-';

			end;
			else do;
				mean_tmp='-';
				median_tmp='-';
				min_tmp='-';
				max_tmp='-';
				q1_tmp='-';
				q3_tmp='-';
				std_tmp='-';
			end;

			*gapnum;
			if a %if "&tablesub" ne "A" %then %do; | c %end; then do;
				if gapcat = 1 then do;
					gaplabel = 'First gap';
					output t12&tablesub.;
				end;
				else if gapcat = 2 | gapcat = 3 then do;
					if gapcat = 2 then gaplabel = 'Second and Subsequent Gaps';
					else if gapcat = 3 then gaplabel = 'Follow-up time after last episode';
					output t13&tablesub.;
				end;
			end;
			if b %if "&tablesub" ne "A" %then %do; | d %end; then do;
				gaplabel = 'All gaps';
				output t14&tablesub.;
			end;

			drop mean min median max q1 q3 std n;
		run;

		%if "&tablesub" = "A" %then %let stratvars = ;
		%if "&tablesub" = "B" %then %let stratvars = sex ;
		%if "&tablesub" = "C" %then %let stratvars = agegroup agegroupnum;
		%if "&tablesub" = "D" %then %let stratvars = race;
		%if "&tablesub" = "E" %then %let stratvars = hispanic;
		%if "&tablesub" = "BC" %then %let stratvars = sex agegroup;
		%if "&tablesub" = "BD" %then %let stratvars = sex race;
		%if "&tablesub" = "BE" %then %let stratvars = sex hispanic;
		%if "&tablesub" = "CD" %then %let stratvars = agegroup race;
		%if "&tablesub" = "CE" %then %let stratvars = agegroup hispanic;


		%do tablenum=12 %to 14;

			/*All Tables - merge in exposure and order*/
			proc sql noprint undo_policy=none;
				create table table_t&tablenum&tablesub. as
				select x.*,
		   			   y.exposure as groupn format=$75. format=$&labellen.. length=&labellen.,
					   y.order,
					   0 as stratsort,
					   0 as stratsort2,
					   y.agegroupfmt
				from t&tablenum.&tablesub. as x
				left join &groupfile. as y
				on x.group = y.group and x.runid = y.runid
				order by order, stratsort, gaplabel descending;
			quit;

			%if "&tablesub" ne "A" & %length(%sysfunc(strip(&tablesub.)))=1 %then %do;

				data table_t&tablenum&tablesub.;
					 set table_t&tablenum&tablesub.;

					 *recode groupn;
					 if missing(%scan(&stratvars.,1)) = 0 then do;
					 	%if "&stratvars" = "sex" %then %do;
							if &stratvars='F' then do; groupn="Female"; stratsort=1; end;
              	   			if &stratvars='M' then do; groupn="Male";   stratsort=2; end;
				   			if &stratvars='O' then do; groupn="Other";  stratsort=3; end;
						%end;
					 	%if %scan(&stratvars.,1) = agegroup %then %do;
							if missing(agegroupfmt)=1 then do;
								groupn=strip(agegroup); 
								stratsort=agegroupnum; 
							end;
							else do;
								%do o = 1 %to &agefmtcnt.;
									%let o1 = %scan(&agefmtorder., &o.);
									if order = &o1. then do;
										groupn=strip(put(agegroup, $age&o1.fmt.));
										stratsort=agegroupnum; 
									end;
								%end;
							end;
						%end;
					 	%if "&stratvars" = "race" %then %do;
 							if &stratvars = 0 then do; groupn = "Unknown"; stratsort=1; end;
							if &stratvars = 1 then do; groupn = "American Indian/Alaska Native"; stratsort=2; end;
							if &stratvars = 2 then do; groupn = "Asian"; stratsort=3; end;
							if &stratvars = 3 then do; groupn = "Black/African American"; stratsort=4; end;
							if &stratvars = 4 then do; groupn = "Native Hawaiian/Other Pacific Islander"; stratsort=5; end;
							if &stratvars = 5 then do; groupn = "White"; stratsort=6; end;
 						%end;
					 	%if "&stratvars" = "hispanic" %then %do;
						    if &stratvars = "N" then do; groupn = "Not Hispanic Origin"; stratsort=1; end;
						    if &stratvars = "Y" then do; groupn = "Hispanic Origin"; stratsort=2; end;
							if &stratvars = "U" then do; groupn = "Unknown"; stratsort=3; end;
						%end;
					end;

					drop agegroupfmt;
				run;

				proc sort data=table_t&tablenum&tablesub.;
					by order stratsort descending gaplabel;
				run;

			%end;

			/*Two level strata*/
			%if "&tablesub" ne "A" & %length(%sysfunc(strip(&tablesub.)))=2 %then %do;

				%let firststrat = %scan(&stratvars., 1);
				%let secondstrat = %scan(&stratvars., 2);

				%let agegroupnum = ;
				%if "&firststrat" = "agegroup" %then %do;
					%let agegroupnum = , agegroupnum , agegroupfmt;
				%end;

				/*Create header row*/
				proc sql noprint;
					create table table_t&tablenum&tablesub._headerrow as
					select distinct runid, 
									group,
									order, 
									stratsort, 
									stratsort2,
									&firststrat.
									&agegroupnum.
					from table_t&tablenum&tablesub.
					where missing(&firststrat.)=0;
				quit;

				%put &secondstrat.;

				data table_t&tablenum&tablesub.;
					 set table_t&tablenum&tablesub. table_t&tablenum.&tablesub._headerrow;

					 *First stratification always sex or age group;
					 *recode groupn;
						 %if "&firststrat" = "sex" %then %do;
						 	if &firststrat='F' then do; groupn="Female"; stratsort=1; end;
              	   			if &firststrat='M' then do; groupn="Male";   stratsort=2; end;
				   			if &firststrat='O' then do; groupn="Other";  stratsort=3; end;
						%end;
						%if "&firststrat" = "agegroup" %then %do;
							if missing(agegroup) = 0 then do;
								if missing(agegroupfmt)=1 then do;
									groupn=strip(agegroup); 
									stratsort=agegroupnum; 
								end;
								else do;
									%do o = 1 %to &agefmtcnt.;
										%let o1 = %scan(&agefmtorder., &o.);
										if order = &o1. then do;
											groupn=strip(put(agegroup, $age&o1.fmt.));
											stratsort=agegroupnum; 
										end;
									%end;
								end;
							end;
						%end;

					if missing(&firststrat.) = 0 and missing(&secondstrat.) =0 then do;
					 	%if "&secondstrat" = "agegroup" %then %do;
							if missing(agegroup) = 0 then do;
								if missing(agegroupfmt)=1 then do;
									groupn=strip(agegroup); 
									stratsort2=agegroupnum; 
								end;
								else do;
									%do o = 1 %to &agefmtcnt.;
										%let o1 = %scan(&agefmtorder., &o.);
										if order = &o1. then do;
											groupn=strip(put(agegroup, $age&o1.fmt.));
											stratsort2=agegroupnum; 
										end;
									%end;
								end;
							end;
						%end;
					 	%if "&secondstrat" = "race" %then %do;
 							if &secondstrat = 0 then do; groupn = "Unknown"; stratsort2=1; end;
							if &secondstrat = 1 then do; groupn = "American Indian/Alaska Native"; stratsort2=2; end;
							if &secondstrat = 2 then do; groupn = "Asian"; stratsort2=3; end;
							if &secondstrat = 3 then do; groupn = "Black/African American"; stratsort2=4; end;
							if &secondstrat = 4 then do; groupn = "Native Hawaiian/Other Pacific Islander"; stratsort2=5; end;
							if &secondstrat = 5 then do; groupn = "White"; stratsort2=6; end;
 						%end;
					 	%if "&secondstrat" = "hispanic" %then %do;
						    if &secondstrat = "N" then do; groupn = "Not Hispanic Origin"; stratsort2=1; end;
						    if &secondstrat = "Y" then do; groupn = "Hispanic Origin"; stratsort2=2; end;
							if &secondstrat = "U" then do; groupn = "Unknown"; stratsort2=3; end;
						%end;
					end;
					drop agegroupfmt;
				run;
					
				proc sort data=table_t&tablenum&tablesub.;
					by order stratsort stratsort2;
				run;

				proc datasets nowarn noprint lib=work;
					delete table_t&tablenum.&tablesub._headerrow;
				quit;

			%end;

			*recode groupnfor table 13 to only print 1x per grouping;
			%if &tablenum.=13 %then %do;
			data table_t&tablenum&tablesub.;
				set table_t&tablenum&tablesub.;
				if gapcat = 2 and stratsort ne 0 then stratsort = stratsort+.5;
			run;
			%end;

		%end; /*loop through tables*/

	%end; /*loop through strata*/

	proc datasets nowarn noprint lib=work;
		delete _bygap_: _allgaps_: _gapstables _distribution t12: t13: t14: ;
	quit;

	%put =====> END MACRO: ms_t5_create_gaps_table v2.1;

%mend ms_t5_create_gaps_table;
