****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_createdistbaselinetable.sas  
*
* Created (mm/dd/yyyy): 05/20/2018
* Last modified: 10/20/2018
* Version: 1.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will create a baseline table, and an optional covariate profile table                                      
*   
*  Program inputs:  

*                                     
*  Program outputs: 
*	-MSOC.&RUNID._baseline&outcohort._&i.;
*	-MSOC.&RUNID._profile._&i.;

*  PARAMETERS:       
*            
*  Programming Notes:                                                                                
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*   1.1		  10/20/18	  AP		Added additional type 4 and mother-infant metrics to baseline table
*
***************************************************************************************************;

%macro ms_createdistbaselinetable(where=, t3out=);

%put =====> MACRO CALLED: ms_createdistbaselinetable v1.1;

	*loop through PeriodID;
	%do i=&PERIODIDSTART. %to &PERIODIDEND.;

		proc sql noprint;
			select count(*) into: nusers
			from DPLocal.&RUNID._ads_&data._&i.
			where &where.;
		quit;

		%if %eval(&nusers.>0) %then %do;

			*************************
			*Create baseline profile*
			*************************;

			%if "&profile." = "y" %then %do;
				data _tempcovar;
					set DPLocal.&RUNID._ads_&data._&i.;
					patient = 1;
					where &where.;
				run;
				proc means data=_tempcovar noprint nway;
					var patient;
					class &groupvar. patid covar1-covar%eval(&numcovars.+&numcomb.);
					id patient;
					output out=_covarprofile(drop=_:) max(patient) = npts
													  sum(patient) = n_episodes;
				run;
					
				proc means data=_covarprofile nway noprint missing;
					var Npts n_episodes;
				    class &groupvar. covar1-covar%eval(&numcovars.+&numcomb.);
				    output out=msoc.&runid._profile&outcohort.&t3out._&i.(drop=_:) sum=;                                  
				run;
			%end;

			***********************
			*Create Baseline table*
			***********************;

			*Strip age_cat of special characters;
			  data _RawData;
				  set DPLocal.&RUNID._ads_&data._&i. (drop = agegroupnum); 
				  by &groupvar. patid indexdt &delnum.;
				  Age_cat=TRANSLATE(Age_cat,"_","-"); 
				  Age_cat=upcase(compress(Age_cat,' <>+-'));
				  patient=1;
				  where &where.;
			  run;

			*Square Age for reporting;
			data _tempage1;
				set infolder.&cohortfile.(keep=agestrat);
					if missing(agestrat) then agestrat='00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+';
				c = countw(agestrat,' ');
				do i = 1 to c;
					age_cat= scan(agestrat, i, ' ');
					Age_cat=TRANSLATE(age_cat,"_","-"); 
				  	Age_cat=upcase(compress(age_cat,' <>+-'));
					output;
				end;
			run;
			proc sql noprint;
				create table _tempage2 as
				select distinct age_cat, . as patient
				from _tempage1
				order by age_cat;
			quit;
			proc transpose data=_tempage2 out=_AgeGroups1(keep=age:) prefix=AGE;
				var patient;
				id age_cat;
			run;

			*Create age group dummies - can be multiple index per patient;
			  proc transpose data=_RawData out=_AgeGroups(keep=&groupvar. patid indexdt Age: &delnum.)  prefix=AGE;
				  var patient;
				  by &groupvar. patid indexdt &delnum.;
				  id Age_cat;
			  run;

			  *Determine missing age groups;
			  proc contents noprint data=_agegroups out=_agecontents(where=(index(upcase(name),'AGE')>0) keep=name);  run;
			  proc contents noprint data=_AgeGroups1 out=_agecontents1(where=(index(upcase(name),'AGE')>0) keep=name); run;
			  proc sort data=_agecontents; by name; run;
			  proc sort data=_agecontents1; by name; run;

			  data _missingagegroups;
			  	merge _agecontents(in=a) _agecontents1(in=b);
				by name;
				if b and not a;
			  run;

		      proc sql noprint;
				select distinct name into: agevars separated by ' '
				from _missingagegroups;
			  quit;

			  %isdata(dataset=_missingagegroups);
			  %if %eval(&nobs.>0) %then %do;
			  data _AgeGroups;
				set _AgeGroups _AgeGroups1(keep=&agevars. obs=0);
			  run;
			  %end;

			  proc datasets nowarn noprint lib=work;
				delete _tempage: _agecontents: _agegroups1;
			  quit;
			  	
			*Create sex dummies - can be multiple index per patient;
			  proc transpose data=_RawData out=_Genders(keep=&groupvar. patid indexdt Sex_: &delnum.)  prefix=Sex_;
				  var patient;
				  by &groupvar. patid indexdt &delnum.;
				  id Sex;
			  run;

			*Create race dummies - can be multiple index per patient;
			  proc transpose data=_RawData out=_races(keep=&groupvar. patid indexdt Race_: &delnum.)  prefix=Race_;
				  var patient;
				  by &groupvar. patid indexdt &delnum.;
				  id race;
			  run;

			*Create hispanic dummies - can be multiple index per patient;
			  proc transpose data=_RawData out=_hispanic(keep=&groupvar. patid indexdt Hispanic_: &delnum.)  prefix=Hispanic_;
				  var patient;
				  by &groupvar. patid indexdt &delnum.;
				  id hispanic;
			  run;

			*Create year dummies - can be multiple index per patient;
			  proc transpose data=_RawData out=_years(keep=&groupvar. patid indexdt year_: &delnum.)  prefix=Year_;
				  var patient;
				  by &groupvar. patid indexdt &delnum.;
				  id year;
			  run;

			%if %eval(&type.=4) %then %do;
			*Create PrePostInd dummies;
			  proc transpose data=_RawData out=_prepostind(keep=&groupvar. patid indexdt Prepostind_: &delnum.)  prefix=Prepostind_;
				  var patient;
				  by &groupvar. patid indexdt &delnum.;
				  id prepostind;
			  run;			
			%end;

			*Merge back dummies;
			data _RawData;
				merge _RawData(drop=patient)
				      _AgeGroups
				      _Genders
					  _years
					  _races
					  _hispanic
					  %if %eval(&type.=4) %then %do;
					  _prepostind
					  %end;
					   ;
				by &groupvar. patid indexdt &delnum.;

				patient=0;

				*create a new patient indicator to count distinct patients;
				if first.patid then patient=1;

				if not first.patid then do;
					array Sexes Sex_:;
					do over Sexes;
						Sexes=.;
					end;

					array races race_:;
					do over races;
						races=.;
					end;

					array hispanics hispanic_:;
					do over hispanics;
						hispanics=.;
					end;
				end;
			run;

			*Process discrete vars;
			  proc means data=_RawData(drop=age age_cat) nway noprint;
				  var patient Age: Sex_: year_: race_: hispanic_:
						%if %eval(&numcovars. ge 1) %then %do; covar: %end;
						%if %eval(&numcomb. ge 1) %then %do; covar: %end;
						%if %eval(&type.=4) %then %do; prepostind_: %end;
						%if "&data." = "mstr_mi" %then %do; exp_: %end;
					;
				  class &groupvar.;
				  output out=_Discrete(drop=_:) sum=;
			  run;

			*Process continuous vars;
			  proc means data=_RawData nway noprint;
				  var /*Followuptime_ITT Followuptime_astreated */
				      Age COMORBIDSCORE 
				      NumAV NumOA NumIP NumIS NumED 
				      NumGeneric NumClass NumRx
						%if %eval(&type.=4) %then %do;
						ga_birth
						%end;
						%if "&data." = "mstr_mi" %then %do;
						ga_first adjusteddisp_pre adjusteddisp_t1 adjusteddisp_t2 adjusteddisp_t3 birth_enroll enroll_diff
						%end;
						;
				  class &groupvar.;
				  output out=_Continuous(rename=_freq_=N_episodes drop=_type_)
				        mean= mean_Age mean_COMORBIDSCORE 
				             mean_NumAV mean_NumOA mean_NumIP mean_NumIS mean_NumED 
				             mean_NumGeneric mean_NumClass mean_NumRx %if %eval(&type.=4) %then %do; mean_ga_birth %end;
							 %if "&data." = "mstr_mi" %then %do;
							 mean_ga_first mean_disp_pre mean_adjusteddisp_t1 mean_adjusteddisp_t2 mean_adjusteddisp_t3 mean_birth_enroll mean_enroll_diff
							 %end;
				         std= std_Age std_COMORBIDSCORE 
				             std_NumAV std_NumOA std_NumIP std_NumIS std_NumED 
				             std_NumGeneric std_NumClass std_NumRx %if %eval(&type.=4) %then %do; std_ga_birth %end;
							 %if "&data." = "mstr_mi" %then %do;
							 std_ga_first std_disp_pre std_adjusteddisp_t1 std_adjusteddisp_t2 std_adjusteddisp_t3 std_birth_enroll std_enroll_diff
							 %end;
						;
			  run;

			*merge the two;
			  data msoc.&RUNID._baseline&outcohort.&t3out._&i.;
				  merge _discrete
				        _continuous;
				  by &groupvar.;
			  run;

			*add missing groups if any; 
     		proc sql noprint;
				create table _MissingGroups as
				%if "&groupvar" = "group" %then %do;
				select distinct (cohortgrp) as &groupvar., 
				%end;
				%if "&groupvar" = "analysisgrp" %then %do;
				select distinct (analysisgrp) as &groupvar., 
				%end;
				0 as patient, 0 as N_Episodes  
				from &groupdefdata.
				where &groupvar. not in
				  (select distinct (&groupvar.) 
				   from msoc.&RUNID._baseline&outcohort.&t3out._&i.)
				;
			quit;
					 
			data msoc.&RUNID._baseline&outcohort.&t3out._&i.;
				set msoc.&RUNID._baseline&outcohort.&t3out._&i.
					_MissingGroups;
				run;
		%end;
		%else %do;
			%put WARNING: No users. Baseline table &RUNID._baseline&outcohort.&t3out._&i. will not be produced.;
		%end;
	%end;

%put NOTE: ******** END OF MACRO: ms_createdistbaselinetable v1.1 ********;

%mend ms_createdistbaselinetable;
