****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: finalizeptsmasterlist.sas  
*
* Created (mm/dd/yyyy): 02/02/2017
* Last modified: 02/12/2019
* Version: 1.11
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*	This program will finalize the numerator dataset by doing final calculation and variables assessment 
*
*  Program inputs:                                                                                   
*	-Dataset with valid numerator observations (_PtsMasterList)
* 
*  Program outputs:                                                                                                                                       
* 	-Dataset with valid numerator observations (_PtsMasterList)
* 
* PARAMETERS:
*
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*   1.1		  05/04/17		AP		Added race_out and hispanic_out cohortfile parameters
*
*   1.2		  10/12/17		AP		Modified to only include age at index date in Type 5 (QRP-446)
*
*	1.3	 	  10/12/17		AP		Added functionality to keep blackout and updated Type 4 PrePostInd 
*									(QRP-319, QRP-388)
*
*	1.4	 	  11/20/17		AP		Removed functionality to keep blackout (QRP-466) 
*
*	1.5	   	  12/28/17	 	AP		Added censoring metrics for Type 1 (QRP-462)
*
*   1.6       03/14/18      RR		Changed Group length to $40 (QRP-495)
*
*   1.7       03/27/18      AP		Changed EpisodeLength to CumEpisodeLength (QRP-516)
*
*   1.8       04/13/18      AP		Modified Type 1 code counting
*									Changed dispensing counts to Adjusted/Raw code counts (QRP-305)
*
*	1.9	   	  05/15/18	 	AP		Added Type 6 analysis, Type 3 Metadata
*									Fixed bug in creation of T3_censor mstr file (DEV-1062)
*
*	1.10	  10/25/18	 	AP		Added Type 4 gestational age of pregnancy
*
*	1.11	  02/12/19	 	AP		Changed MsReqID to ReqID
*
***************************************************************************************************;

%macro ms_finalizeptsmasterlist();
	
%put =====> MACRO CALLED: ms_finalizeptsmasterlist v1.11;

	****************************************;
	* Type 1, 2, and 6 cohort definition	;
	****************************************;
	%if %eval(&type.=1 | &type.=2 | &type.=6) %then %do;

    	proc datasets library=work nolist nowarn;
    	delete _GroupUtilFile;
    	quit;

		%if %eval(&type=1) %then %do; *TYPE 1 ONLY;

	        data _PtsMasterList;
		        set _PtsMasterList;
		        by Patid IndexDt;
		        if T1CohortDef ='01' then do;
		            if first.PatId;
		        end;

				format Group $40.;
			    Group="&ITGROUP."; 
			    Year=Year(Indexdt);
			    Type=input("&type.",best.);

				FEventDt = .;
				month=month(indexdt);

				%let enddate_forcensor = .;
				%if &censor_qryend = 1 %then %do;
					%let enddate_forcensor = &enddate;
				%end;
				%let dpdate_forcensor = .;
				%if &censor_dpend = 1 %then %do;
					%let dpdate_forcensor = &DP_MaxDate;
				%end;
				%let deathdt_forcensor = .;
				%if &censor_dth = 1 %then %do;
					%let deathdt_forcensor = deathdt;
				%end;

				timetocensor=Min(Enr_End,&deathdt_forcensor., &enddate_forcensor.,&dpdate_forcensor. )-IndexDt +1;

				/*Enumerate censoring criteria*/

				/*initialize censoring variables*/
				cens_elig = 0;
				cens_dth = 0;
				cens_dpend = 0;
				cens_qryend = 0;

				EpisodeEndDt_Censor = min(Enr_End,&deathdt_forcensor., &enddate_forcensor.,&dpdate_forcensor.);
				
				/*Censored due to enrollment*/
				If EpisodeEndDt_Censor = Enr_End then cens_elig = 1; else cens_elig = 0;

				/*Censored due to death*/
				If EnrEpisodeCensoredAtDeath = 1 and EpisodeEndDt_Censor = Deathdt then cens_dth = 1; else cens_dth = 0;

				/*Censored due to query end*/
				if EpisodeEndDt_Censor = &enddate. then cens_qryend = 1; else cens_qryend = 0;

				/*Censored due to DP Max*/
				%if &Censor_DPEnd. = 1 %then %do;
					if EpisodeEndDt_Censor = &dp_maxdate. then cens_dpend = 1; else cens_dpend = 0;

						%if &DPMax_Adjust. =1 %then %do;
							Cens_QRYEND = 0; /*Reset Cens_qryend because we adjusted enddate to equal dp_maxdate*/
						%end;
				%end;

				/*Adjust enr_end for death (for denoms)*/
				if EnrEpisodeCensoredAtDeath and DeathDt <= Enr_End then Enr_End=DeathDt;

				format EpisodeEndDt_Censor OrigEpisEndDt date9.;

		        keep Group Type PatId Enr_start Enr_End year month IndexDt EpisodeEndDt indexlook RxAmt RxSup NumDispensing FEventDt
						cens_elig cens_dth cens_dpend cens_qryend deathdt timetocensor;
	        run;

			*Because episodes are not being created, we only want to count codes with an Adate on index date;
			%let KeepPartBf = N;
		%end;
		%else %do; *TYPE 2 and 6;

			%if %eval(&type.=2) %then %do;
				*Set first event date among all episodes for patients;
			    data _PtsMasterList;
					merge _PtsMasterList
				          _FirstEventDtInEpisode;
				    by PatId;
			    run;

				*Set events and final variables;
				data _PtsMasterList;
					merge _PtsMasterList(in=a)
				          _EPisodesWithEvents(in=b);
				    by Patid IndexDt;
				    if a;
				run;
			%end;

			data _PtsMasterList;
				format Group $40.;
				set _PtsMasterList;
			    by Patid IndexDt;

				%if %eval(&type.=6) %then %do;
					FEventDt = .;
					FEventDt_Any = .;
					NumEvents = .;
				%end;
		
		        tte=Min(EpisodeEndDt,Enr_End,FEventDt)-IndexDt -&BLACKOUTPER. +1;

		        if T&type.CohortDef ='01' then do;
		            if first.PatId;
		        end;
		        if T&type.CohortDef ='03' then do;
		            if FEventDt_Any and IndexDt > FEventDt_Any then delete;
		        end;	        

			    Group="&ITGROUP."; 
			    Year=Year(Indexdt);
			    Type=input("&type.",best.);

		        if &itt.=1 then EpisodeType="ITT";
		        else EpisodeType="EPI";

				/*Enumerate censoring criteria*/

				/*initialize censoring variables*/
				cens_elig = 0;
				cens_dth = 0;
				cens_dpend = 0;
				cens_qryend = 0;
				cens_episend = 0;	
				cens_spec = 0;
				cens_event = 0;

				/*Episode is not adjusted for events, so need to consider true censoring date*/
				EpisodeEndDt_Censor = min(EpisodeEndDt,Enr_End,FEventDt);

				/*Censored due to event*/
				if EpisodeEndDt_Censor = FEventDt then do;
					Event_flag = 'Y';
					cens_event = 1;
				end;
				else do;
					Event_flag = 'N';
					cens_event = 0;
				end;

				/*Censored due to enrollment*/
				If EpisodeEndDt_Censor = Enr_End then cens_elig = 1; else cens_elig = 0;
					/*Reset if episode truncated due to death and original enrollment did not occur on death date*/
					if cens_elig = 1 and EnrEpisodeCensoredAtDeath = 1 and enr_end_equal_death ne 1 then cens_elig = 0;

				/*Censored due to death*/
				If EnrEpisodeCensoredAtDeath = 1 and EpisodeEndDt_Censor = Deathdt then cens_dth = 1; else cens_dth = 0;

				/*Censored due to query end*/
				if EpisodeEndDt_Censor = &enddate. then cens_qryend = 1; else cens_qryend = 0;

				/*Censored due to DP Max*/
				%if &Censor_DPEnd. = 1 %then %do;
					if EpisodeEndDt_Censor = &dp_maxdate. then cens_dpend = 1; else cens_dpend = 0;

						%if &DPMax_Adjust. =1 %then %do;
							Cens_QRYEND = 0; /*Reset Cens_qryend because we adjusted enddate to equal dp_maxdate*/
						%end;
				%end;

				/*Censored due to requester defined criteria*/
				if trunkdate and EpisodeEndDt_Censor = trunkdate then cens_spec = 1; else cens_spec = 0;

				/*Censored due to episode end date*/
					/*No adjustments have been made*/
					if (OrigEpisEndDt = EpisodeEndDt_Censor) then cens_episend = 1;
					else if not missing(maxepisdur) and EpisodeEndDt_Censor = IndexDt + maxepisdur -1 then cens_episend = 1;
					else cens_episend = 0;

				/*Adjust enr_end for death (for denoms)*/
				if EnrEpisodeCensoredAtDeath and DeathDt <= Enr_End then Enr_End=DeathDt;

				format EpisodeEndDt_Censor OrigEpisEndDt LastLookFollowedDt date9. EndFollowUpType $20.;

				timetocensor = tte;
				episodelength = tte;
				month=month(indexdt);

				* This will be applied later for Surveillance analysis;
				LastLookFollowed=0;
				LastLookFollowedDt=.;
				EndFollowUpType="";

		        keep Group Type PatId Enr_start Enr_End IndexDt EpisodeEndDt NumEvents FEventDt tte year month EpisodeType blackoutper: IndexLook
					cens_elig cens_dth cens_dpend cens_qryend cens_episend cens_spec cens_event deathdt Event_flag timetocensor minepisdur LastLookFollowed:
					%if %eval(&type.=6) %then %do;
					INCEXL PrevInc code productapprovaldate productmarketingdate uptakedate otherproductdate EpisodeEndDt_Censor episodelength
					%end;
					;
			run;

			%let KeepPartBf = Y;
		%end;	

		*Summarize utilization overlapping only FINAL episodes;
		%ms_shaveoutside(reffile=_PtsMasterList,
	                     refstart=IndexDt,
	                     refend=EpisodeEndDt,
	                     KeepPartBf=&keeppartbf.,
	                     ToShaveFile=_GroupIndex,
	                     ToShaveStart=Adate,
	                     ToShaveEnd=ExpireDt,
	                     outfile=_GroupUtilFile);


		proc means data=_GroupUtilFile nway noprint;
			var RxSup RxAmt NumDispensing;
		    class PatId IndexDt;
		    output out=_GroupUtilFile(rename=_freq_=AdjustedDisp drop=_Type_) sum(RxSup RxAmt NumDispensing)=TotRxSup TotRxAmt RawDisp;
		run;	
	
    	proc sort nodupkey data=_agegroups;
    		by PatId IndexDt;
    	run;

    	data _PtsMasterList;
	    	merge _PtsMasterList (in=a)
	    	      _GroupUtilFile (in=b)
	    		  _AgeGroups;
	    	by PatId IndexDt;
    	if a or b;*defensive;

		%if "&agedatecohort" = "Y" %then %do;	
			TotRxSup=0;
			TotRxAmt=0;
			AdjustedDisp=0;	
			RawDisp=0;
			%if %eval(&type.=2) %then %do;
			EPISODETYPE = 'ITT';
			%end;
		%end;
    	run;

		*Type 6 - finalize dispensings;
		%if %eval(&type.=6) %then %do;
			*keep dispensings overlapping episodes and 
			 remove dispensings that do not meet inclusion/exclusion criteria;
			%ms_shaveoutside(reffile=_PtsMasterList,
	                         refstart=IndexDt,
	                         refend=EpisodeEndDt,
	                         KeepPartBf=Y,
	                         ToShaveFile=_GroupIndex2,
	                         ToShaveStart=Adate,
	                         ToShaveEnd=ExpireDt,
	                         outfile=_GroupUtilFile);

			proc sql noprint undo_policy=none;
				create table _dispensings as
				select distinct util.*
					, pts.age
					, pts.agegroup /*Age group at index date*/
					, pts.agegroupnum
					, pts.minagedate
					, pts.maxagedate
					, &startfollowup. as StudyStartDate format=date9.
					, &enddate. as StudyEndDate format=date9.
					, pts.birth_date
					, pts.sex
					, pts.race
					, pts.hispanic
					, pts.zip3
					, pts.state
					, pts.zip_uncertain
					, pts.hhs_reg
					, pts.cb_reg
					, "&ITGROUP." as Group format=$40. length 40
				from _grouputilfile as util, _ptsmasterlist as pts
				where util.patid = pts.patid and util.indexdt = pts.indexdt and pts.INCEXL=1
				order by patid, indexdt, adate;
			quit;

			/*remove labels*/
			proc datasets lib=work memtype=data nolist nowarn;
		    modify _dispensings; 
		    attrib _all_ label=' '; 
		    run;
		%end; *Type 6;

    %end;  *TYPE 1, 2, and 6;

	****************************************;
	* Type 3 cohort definition 				;
	****************************************;
	%if %eval(&type=3) %then %do;

		*Create a master file for Censor data;
		data _PtsMasterList_Censor;
			set _PtsMasterList;
		run;

		%macro finalizeT3Master(suffix=, censorcondition=);

			*Keep only valid IndexDt in master file;
			data _PtsMasterList&suffix.;
		        merge _PtsMasterList&suffix.(in=a)
		              _Events(in=b);
		        by PatId IndexDt;
		        if a;

				format Group $40.;
			    Group="&ITGROUP."; 
			    Year=Year(Indexdt);
			    Type=input("&type.",best.);

		        if first.patid then do;
		            Pt_Analysis_Episode_Num=0; 
		            Pt_Exposure_Episode_Num=0;
		        end;

		        Pt_Exposure_Episode_Num=Pt_Exposure_Episode_Num+1;

		        if &ExpPeriodStartDt. <= indexdt <= &ExpPeriodEndDt.;

		        *01: Cohort includes only the first exposure during the query period;
		        if T3CohortDef ='01' then do;
		            if first.PatId;
		        end;

				TTC=.;
				CensorDeath=0;
		        CensorEnrol=0;
		        if MaxEligtDt > ENR_END then do;
		            if EnrEpisodeCensoredAtDeath then CensorDeath=1;
		            else CensorEnrol=1;
					TTC=ENR_END-Indexdt;
				end;

				*Analysis cohort
					1. Analysis cohort including incomplete post-index enrollment criteria (censor)
					2. Analysis cohort with full all post-index enrollment criteria met;

		        AnalysisCohort=0; 
	   			if FEventDt ne . and MinEnrolHOIMet=1 and MaxEligtEnrAftDt <= ENR_END &censorcondition. then do;
		            DAYS_FROM_EXPOS_TO_EVENT=FEventDt-IndexDt;
		            if WinOfEvent="CTRL" then DAYS_FROM_EXPOS_TO_CTRL_EVENT=DAYS_FROM_EXPOS_TO_EVENT;
		            if WinOfEvent="RISK" then DAYS_FROM_EXPOS_TO_RISK_EVENT=DAYS_FROM_EXPOS_TO_EVENT;
		            AnalysisCohort=1;
		            Pt_Analysis_Episode_Num=Pt_Analysis_Episode_Num+1;
		        end;    

				if AnalysisCohort=0 then do;
					FEventDt=.;
					WinOfEvent="";
				end; 

		        POSTENR_EXPOSURE=Enr_End-IndexDt;
				month=month(indexdt);

		        retain Pt_Analysis_Episode_Num Pt_Exposure_Episode_Num;

		        keep Group Type PatId Enr_start Enr_End IndexDt FEventDt DAYS_FROM: year month WinOfEvent MinEnrolHOIMet 
					 TTC age birth_date sex race hispanic AgeGroup: Censor: AnalysisCohort Pt_: POSTENR_EXPOSURE zip: state hhs_reg cb_reg;
			run;

		%mend;

		%finalizeT3Master(suffix=, censorcondition= %str(and CensorDeath=0 and CensorEnrol=0));
		%finalizeT3Master(suffix=_censor, censorcondition=);

        *Delete patients with multiple NDC on the incident exposure date;
        %if %str(&ExclOnSameDay)=%str(Y) %then %do; 
            data _PtsMasterList
                 _SameDayDeleted;
            merge _PtsMasterList(in=a drop=TTC)
                  _SameDayNDCs(in=b keep=PatId Rxdate rename=Rxdate=indexDt);
            by PatId IndexDt;
            if a;
            if b then output _SameDayDeleted;
            else output _PtsMasterList;
            run;

			data _PtsMasterList_Censor;                 
            merge _PtsMasterList_Censor(in=a)
                  _SameDayNDCs(in=b keep=PatId Rxdate rename=Rxdate=indexDt);
            by PatId IndexDt;
            if a and not b then output _PtsMasterList_Censor;
            run;
        %end;
		%else %do;
			data _SameDayDeleted;
			set _PtsMasterList(keep=PatId IndexDt);
			if _N_=0 then output;
			run;
		%end;

		*Compute MetaData for this group;
		data _Type3Info;
			merge infolder.&TYPE3FILE.(where=(strip(group)="&itgroup.")) 
				  infolder.&COHORTFILE.(where=(strip(group)="&itgroup.") rename=cohortgrp=group keep=cohortgrp enrdaysaftind);
			by group;
		run;

	    data _metadata;
	    	set _Type3Info;
	    	format T3SURVSTARTDATE T3ENDOFUPDATE ExpPeriodStartDt ExpPeriodEndDt date9.;

		    TimePeriod=&PERIODIDSTART.;
		    RequiredFUPPer=max(0,T3RiskTo,T3CtrlTo,EnrDaysAftInd,&enrdaysaftind.);

		    ExpPeriodStartDt=T3SURVSTARTDATE;
		    ExpPeriodEndDt=INTNX('Month', T3ENDOFUPDATE-RequiredFUPPer,0, 'begin')-1;

			ReqID="&REQID.";
			RunID="&RUNID.";
		    keep group TimePeriod ExpPeriodStartDt ExpPeriodEndDt T3SURVSTARTDATE T3ENDOFUPDATE ReqID RunID;
	    run;

	    %if &PERIODIDSTART. > 1 %then %do;
			data _IncrDates(drop=ExpPeriodEndDt T3ENDOFUPDATE); 
			format ExpPeriodStartDt date9.;
	        set infolder.&T3METADATA.(where=(strip(group)="&itgroup."));  
			ExpPeriodStartDt=INTNX('Month', ExpPeriodEndDt+1, 0, 'begin');
			run;

	        data _metadata;
	        merge _metadata(drop=ExpPeriodStartDt)
	              _IncrDates;
	        by group;
			TimePeriod=&PERIODIDSTART.;
	        *ExpPeriodStartDt=&ExpPeriodStartDt.;
			keep group TimePeriod ExpPeriodStartDt ExpPeriodEndDt T3SURVSTARTDATE T3ENDOFUPDATE ReqID RunID;
	        run;

	    %end;

		%if %eval(&group. = 1) %then %do;
			data msoc.&runid._Metadata_for_time_period_&PERIODIDSTART.;
			retain ReqID RunID GROUP TimePeriod T3SURVSTARTDATE ExpPeriodStartDt ExpPeriodEndDt T3ENDOFUPDATE;
			set _metadata;
			run;
		%end;
		%else %do;
			proc append base=msoc.&runid._Metadata_for_time_period_&PERIODIDSTART.
		                data=_metadata force;
		    run;
		%end;

	%end; *TYPE 3;

	****************************************;
	* Type 4 cohort definition 				;
	****************************************;
	%if %eval(&type=4) %then %do;

		data _PtsMasterList (drop=lEpisodeEndDt eligdays episode expiredt minenrolmet PrePostPriority priority:);
        set _PtsMasterList;
        by Patid IndexDt;		

		format Group $40.;
	    Group="&ITGROUP."; 
	    Type=input("&type.",best.);

		* If there are overlapping episodes, truncate the episode and set the start date  
		  at the day following the end date of the preceding episode;		
	    lEpisodeEndDt = lag(EpisodeEndDt);
	    if first.PatId then lEpisodeEndDt = .;
	    if EpisodeStartDt <= lEpisodeEndDt then do;
			EpisodeStartDt = lEpisodeEndDt + 1;
			IndexDt=EpisodeStartDt;
		end;
		FEventDt = .;
		year=year(EpisodeEndDt);
		month=month(EpisodeEndDt);
        run;

		proc sort data=_PtsMasterList;
			by patid episodeenddt;
		run;

    	data _PtsMasterList;
    	merge _PtsMasterList (in=a)
    		  _AgeGroups(rename=indexdt=EpisodeEndDt);
    	by PatId EpisodeEndDt;
    	if a;
		run;

		proc sort data=_Deliveries;
		by Patid EpisodeStartDt;
		run;

		* If there are overlapping episodes, truncate the episode and set the start date  
		  at the day following the end date of the preceding episode;	
		data _Deliveries;
		set _Deliveries;
		by Patid EpisodeStartDt;					
	    lEpisodeEndDt = lag(EpisodeEndDt);
	    if first.PatId then lEpisodeEndDt = .;
	    if EpisodeStartDt <= lEpisodeEndDt then EpisodeStartDt = lEpisodeEndDt + 1;
		drop lEpisodeEndDt;
		run;

		* Make sure the it is the correct episode start date that is in _PtsMasterList;
		proc sort data=_PtsMasterList;
		by PatId DelNum;
		run;

		proc sort data=_Deliveries;
		by PatId DelNum;
		run;

		data _PtsMasterList;
		merge _PtsMasterList(in=a drop=EpisodeStartDt)
			  _Deliveries (in=b keep=PatId DelNum EpisodeStartDt);
		by Patid DelNum;
		if a;
		IndexDt=EpisodeStartDt;
		Duration=EpisodeEndDt - EpisodeStartDt;
	    Has3Trim=0;
	    if Duration>=181 then Has3Trim=1;
		format PrePostInd $10.;

		PrePostInd="NONE";
		if HadNone ne 1 then do;
			if duration <=258 then PrePostInd = "PRETERM";
			else if 259 <= duration <=280 then PrePostInd = "TERM";
			else if duration >=281 then PrePostInd = "POSTTERM";
		end;

		*Gestational length of pregnancy;
		ga_birth = int(((duration) / 7)+1); /*Pregnancy should always be >0 days*/
		run;

		data _Deliveries;
		merge _Deliveries(in=a)
			  _PtsMasterList (in=b keep=PatId DelNum duration);
		by PatId DelNum;
		format PrePostInd $10.;

		IndexDt=EpisodeStartDt;
		Duration=EpisodeEndDt - EpisodeStartDt;

		PrePostInd="NONE";

		if HadNone ne 1 then do;
		if . < duration <=258 then PrePostInd = "PRETERM";
		else if 259 <= duration <=280 then PrePostInd = "TERM";
		else if duration >=281 then PrePostInd = "POSTTERM";
		end;
		Eligible="N";
		if b then Eligible="Y";
		run;

		proc sort data=_PtsMasterList; 
		by patid ADate; 
		run;

		proc sort data=_GroupStartCodes; 
		by patid ADate; 
		run;

		data _PrePostCodes;
		merge _PtsMasterList(in=a keep=patid adate delnum)              
	          _GroupStartCodes;
		by PatId ADate;
		if a;
		if not missing(PriorityGroup1) and not missing(PriorityGroup2);
		keep PatId Group PriorityGroup1 PriorityGroup2 PregdurCode DelNum;
		run;

	%end;	*TYPE 4

	****************************************;
	* Type 5 cohort definition 				;
	****************************************;
	%if %eval(&type=5) %then %do;

    	proc datasets library=work nolist nowarn;
    	delete _GroupUtilFile _PtsMasterList _dispensings;
    	quit;
	
		data _PtsMasterList;
	        set _PtsMasterList2(rename=indexdt=episodestartdt);
	        by Patid episodestartdt;
	   
	        tte =Min(EpisodeEndDt,Enr_End)-episodestartdt +1;

			StudyStartDate = &startfollowup.;
			StudyEndDate = &enddate.;

			format Group $40.;
   			Group="&ITGROUP."; 
		    Type=input("&type.",best.);

			length episodenum episode_cens_elig episode_cens_dth episode_cens_dpend episode_cens_qryend episode_cens_episend episode_cens_spec 4;

			if first.patid then do;
				episodenum = 1;
				indexdt = episodestartdt;
			end;
			else do;
				episodenum = episodenum + 1;
				indexdt = indexdt;
			end;

			retain episodenum indexdt;

			/*Enumerate censoring criteria*/

			/*initialize censoring variables*/
			episode_cens_elig = 0;
			episode_cens_dth = 0;
			episode_cens_dpend = 0;
			episode_cens_qryend = 0;
			episode_cens_episend = 0;	
			episode_cens_spec = 0;

			/*Consider true censoring date*/
			EpisodeEndDt_Censor = min(EpisodeEndDt,Enr_End, deathdt);

			/*Max Eligibility*/
			MinEligDate = min(enr_end, deathdt, &enddate.);

			/*Censored due to enrollment*/
			If EpisodeEndDt_Censor = Enr_End then episode_cens_elig = 1; else episode_cens_elig = 0;

			/*Censored due to death*/
			If EnrEpisodeCensoredAtDeath = 1 and EpisodeEndDt_Censor = Deathdt then episode_cens_dth = 1; else episode_cens_dth = 0;

			/*Censored due to query end*/
			if EpisodeEndDt_Censor = &enddate. then episode_cens_qryend = 1; else episode_cens_qryend = 0;

			/*Censored due to DP Max*/
			%if &Censor_DPEnd. = 1 %then %do;
				if EpisodeEndDt_Censor = &dp_maxdate. then episode_cens_dpend = 1; else episode_cens_dpend = 0;

					%if &DPMax_Adjust. =1 %then %do;
						episode_Cens_QRYEND = 0; /*Reset Cens_qryend because we adjusted enddate to equal dp_maxdate*/
					%end;
			%end;

			/*Censored due to requester defined criteria*/
			if trunkdate and EpisodeEndDt_Censor = trunkdate then episode_cens_spec = 1; else episode_cens_spec = 0;

			/*Censored due to episode end date*/
				/*No adjustments have been made*/
				if (OrigEpisEndDt = EpisodeEndDt_Censor) then episode_cens_episend = 1;
				else episode_cens_episend = 0;

			/*Adjust enr_end for death*/
			if EnrEpisodeCensoredAtDeath and DeathDt <= Enr_End then Enr_End=DeathDt;

			format EpisodeEndDt_Censor OrigEpisEndDt studystartdate studyenddate indexdt MinEligDate date9.;
			label episodestartdt='Episode Start Date';

	        keep Group PatId Type studystartdate studyenddate indexdt episodenum episodestartdt episodeenddt episode_cens_: tte MinEligDate;
			run;

			*Summarize utilization overlapping only FINAL episodes;
			%ms_shaveoutside(reffile=_PtsMasterList,
	                         refstart=EpisodeStartdt,
	                         refend=EpisodeEndDt,
	                         KeepPartBf=Y,
	                         ToShaveFile=_GroupIndex2,
	                         ToShaveStart=Adate,
	                         ToShaveEnd=ExpireDt,
	                         outfile=_GroupUtilFile);

			proc means data=_GroupUtilFile nway noprint;
				var RxSup NumDispensing;
			    class PatId EpisodeStartdt;
			    output out=_GroupUtilFile_sum(rename=_freq_=AdjustedDisp drop=_Type_) sum(RxSup NumDispensing)=TotRxSup RawDisp;
			run;

		/*Merge demographics at index date*/
		proc sql noprint undo_policy=none;
			create table _PtsMasterList as
			select x.*
				, y.birth_date
				, y.sex
				, y.race
				, y.hispanic
				, y.age
				, y.agegroup
				, y.agegroupnum
				, y.minagedate
				, y.maxagedate
				, y.zip3
				, y.state
				, y.hhs_reg
				, y.cb_reg
				, y.zip_uncertain
			from _PtsMasterList as x
			left join _agegroups as y
			on x.patid = y.patid and x.indexdt = y.indexdt
			order by x.patid , x.EpisodeStartdt DESCENDING  ;
		quit;

		proc sort data=_GroupUtilFile_sum;
    		by PatId DESCENDING EpisodeStartdt;
    	run;	

		data _PtsMasterList;
	    	merge _PtsMasterList (in=a)
	    	      _GroupUtilFile_sum (in=b);
	    	by PatId DESCENDING EpisodeStartdt;
	    	if a or b;*defensive;
		run;

		data _PtsMasterList(drop=MinEligDate LEpiStart);
			set _PtsMasterList;
			by PatId DESCENDING EpisodeStartdt;

			length gapnum Mntsfromstart 4;

			*calculate GapLength;
			if first.patid then do;
				*note: when elgibility = end of episode, gaplength = -1;
				gaplength = MinEligDate - EpisodeEndDt-1;
				gapnum = 999;
				LEpiStart = EpisodeStartdt;
			end;
			else do;
				gaplength = LEpiStart - EpisodeEndDt-1;
				LEpiStart = EpisodeStartdt;
				gapnum = EpisodeNum;
			end;
			retain LEpiStart;	

			/*Calculate months from study start*/
			Mntsfromstart = intck('month', StudyStartDate, EpisodeStartdt, 'c')+1;
    	run;

		/*Calculate cumulative time at risk and episodegap*/
		proc means data=_PtsMasterList nway noprint;
			var tte;
			class patid;
			output out=_episodelength(drop=_:) sum(tte)=CumEpisodeLength;
		run;

		data _PtsMasterList(rename=tte=episodelength);
			merge _PtsMasterList
				_episodelength;
			by patid;

			year=year(indexdt);
			FEventDt = .;
		run;

		/*Finalize _dispensings*/
		proc sql noprint undo_policy=none;
			create table _dispensings as
			select distinct util.*
				, pts.indexdt
				, pts.age
				, pts.agegroup
				, pts.agegroupnum
				, pts.minagedate
				, pts.maxagedate
				, &startfollowup. as StudyStartDate format=date9.
				, &enddate. as StudyEndDate format=date9.
				, pts.episodenum
				, pts.birth_date
				, pts.sex
				, pts.race
				, pts.hispanic
				, pts.zip3
				, pts.state
				, pts.zip_uncertain
				, pts.hhs_reg
				, pts.cb_reg
			from _grouputilfile as util, _ptsmasterlist as pts
			where util.patid = pts.patid and util.episodestartdt = pts.episodestartdt
			order by patid, episodenum;
		quit;

		proc means data=_dispensings noprint nway;
			var rxsup;
			class patid episodenum;
			output out = _dispensings_sum(rename=_freq_=Episodedispensing drop=_type_) sum(rxsup)=episoderxsup;
		run;

		data _dispensings;
			merge _dispensings(rename=adate=disp_rxdate rename=rxsup=disp_rxsup)
				_dispensings_sum;
			by patid episodenum;

			keep StudyStartDate StudyEndDate group patid birth_date sex race hispanic zip3 state hhs_reg cb_reg zip_uncertain agegroup: minagedate maxagedate indexdt episodenum
					episodedispensing episoderxsup disp_rxdate disp_rxsup;
		run;

		/*remove labels*/
		proc datasets lib=work memtype=data nolist nowarn;
	    modify _dispensings; 
	    attrib _all_ label=' '; 
	    run;

	%end; *TYPE 5;

	*Remove all labels;
    proc datasets lib=work memtype=data nolist nowarn;
    modify _PtsMasterList; 
    attrib _all_ label=' '; 
    run;

%put NOTE: ********END OF MACRO: ms_finalizeptsmasterlist v1.11********;

%mend ms_finalizeptsmasterlist;
