****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_t5_output_report.sas  
*
* Created (mm/dd/yyyy): 10/24/2017
* Last modified: 02/05/2019 
* Version: 2.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE: Produces formatted Type 5 report
*                                       
*  Program inputs:                                                                                   
* 
*  Program outputs:                                                                                                                                       
*  	-&requestID_t5_report.pdf
* 
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*   1.1		  01/23/18   AP			Added group identifier variable to output multiple groups in
*									1 folder, shortened output dataset names (QRPL-177)
*
*   1.2		  03/15/18   AP			Changed table 3 title, updated width of table 5 gapnum (QRPL-188, QRPL-191)
*
*   1.3	 	  03/30/18   AP			Changed tables 3 and 4 title (QRPL-203)
*
*   1.4	 	  04/25/18   AP			Replaced Dispensing to AdjustedCodeCount (QRPL-215)
*
*   1.5       07/17/18   RT			updated to output the reports by user's choice also most of the tables
*                                   are changed to output multiple groups in one report.
*
*   2.0       01/03/19   AP			Modified for integration into QRP
*
*   2.1       02/05/19   AP			Added Figure 5
*
***************************************************************************************************;

%macro ms_t5_output_report();

	%put =====> MACRO CALLED: ms_t5_output_report v2.1;

	ods select all;
	ods listing close;
	ods noresults;
	ods pdf file="&msoc./&requestid._t5_report.pdf" startpage = yes dpi=200 NOGTITLE pdftoc=1;
	ods escapechar="^";
	options nodate nonumber nocenter orientation=portrait;

	&report_header.;

	/**************************************************************************************************
	 * Table of Contents													
	 ***************************************************************************************************/
	ods proclabel = "Table of Contents";

	proc report data = toc nofs nowd headline headskip split="*" contents=''
	   style(header)=[rules=none background=white font_weight=bold font_size=8pt fontfamily=arial vjust=b bordertopcolor=black borderbottomcolor=black] split='*'
	   style(report)=[rules=none frame=box fontfamily=arial cellpadding =&line_spacing.pt];
			
	   columns ("^S={just=c background=ligr borderrightcolor=ligr} Table of contents" table caption);			
	   define table / order=data ' '
				      style(column)=[rules=none font_size=8pt font_weight=bold fontfamily=arial width= 1.1in just=L] ;
	   define caption / order=data  ' '
				        style(column)=[font_size=8pt fontfamily=arial width=6.5in just=L] ;
	run;
	
	/**************************************************************************************************
	* Baseline Table											
	***************************************************************************************************/

	%if "&output_baselinetable" = "Y" %then %do;

		%macro baselineprocreport(data=, title=);
			proc report data=&data. nofs nowd contents=''
				style(header)=[rules=none background=white font_weight=bold font_size=8pt fontfamily=arial vjust=b bordertopcolor=black borderbottomcolor=black] split='*'
				style(report)=[rules=none frame=box cellpadding =&line_spacing.pt];

				column ("^S={just=l}&title." (vartype _name_ grouper 
					   ("^S={background=ligr borderrightcolor=ligr}" _label_ ) ("^S={background=ligr borderleftcolor=ligr}" &group._a &group._b))) ;
				define vartype / noprint;
				define _name_ / noprint;
				define grouper / order noprint order=data '';
				define _label_ / display 'Characteristic^{super 1}' 
					style(column)=[font_size=8pt fontfamily=arial indent=10 width=3.05in]  style(header)=[just=L background=white borderbottomcolor=black indent=0];
				define &group._a / display 'N/Mean'  
					style(column)=[font_size=8pt fontfamily=arial width=.85in just=R] style(header)=[just=R background=white borderbottomcolor=black];
				define &group._b / display '%/Std Dev^{super 2}'  
					style(column)=[font_size=8pt fontfamily=arial width=.85in just=R] style(header)=[just=R background=white borderbottomcolor=black];
						
				/*Add header line*/
				compute before grouper / style=[background=ligr fontfamily=arial font_size=8pt just=L font_weight=bold bordertopcolor=black borderbottomcolor=black];
					length text $100;
					if grouper = "Patient Characteristics" then do; 
						text='Demographics'; 
						num=100;
					end;
					else if grouper = "Recorded history of:" then do; 
						text='Recorded history of:'; 
						num=100; 
					end;
					else if grouper = "History of use:" then do; 
						text='History of use:';
						num=100; 
					end; 
					else if grouper = "Health Service Utilization Intensity:" then do;
						text='Health Service Utilization Intensity:';
						num=100;
					end; 	
					else if grouper = "Mahalanobis distance" then do;
						text='';
						num=100;
					end; 	
					else do; 
						text = "";
						num=0;
					end;
					line text $Varying. num; 
				endcomp;

					/*Italicize covariates*/
				
					%if %length(&highlight_vars.) > 0 %then %do;
						compute _label_;
						if upcase(_name_) in (&vars_to_italicize.) then do;
							call define(_row_,'style','style={fontstyle=italic}');
						end;
						endcomp;
					%end;

					/*footnote in table*/
					compute after / style=[just=L fontfamily=arial font_size=8pt bordertopcolor=black];
						line '';
						line "^{super 1}All metrics based on total number of unique patients";
						line "^{super 2}Value represents standard deviation where no % follows the value";
					endcomp;
			run;

		%mend baselineprocreport;
		
        /*Table lettering*/
		%let tablecount = 1;
		%let tableletter = a;	
       
		/*loop through each look*/
		%do i = %eval(&look_start) %to %eval(&look_end);

			%let displayperiodid = ;
			%if %eval(&look_start) ne %eval(&look_end) %then %let displayperiodid = (MP &i. );

            *loop through groups;
			%do iden=1 %to &num_allgroups.;

				data _null_;
					set &groupfile.(where=(order=&iden.));
						call symputx('group', group);
					 	call symputx('expos', exposure);
					 	call symputx('highlight_vars', highlight_vars);
						call symputx('alphabetical_covarsort', upcase(alphabetical_covarsort));
				run;

				
				%let covarsort = sortorder2 _name_;
				%if "&alphabetical_covarsort" = "Y" %then %let covarsort = _label_;

				proc sort data=tab_agg_l&i._&iden. sortseq=linguistic(numeric_collation=on); 
					by sortorder &covarsort.;
				run;		

			  	%tableletter;
		
			  	/*Create list of covariates to italicize*/
		     	%if %length(&highlight_vars.) > 0 %then %do;
			    	%create_comma_charlist(inlist=&highlight_vars., outlist=vars_to_italicize);
			    	%put &vars_to_italicize.;
		      	%end;
			 
			  	ods proclabel = "Table 1&tableletter.";
			  	&report_header.;
			  	%baselineprocreport(data=tab_agg_l&i._&iden., title=%quote(Table 1&tableletter.: Baseline table &displayperiodid.(&expos.)));
    		%end;
		   							
		%end; /* look */
	%end; *if baseline output = Y;

	/**************************************************************************************************
	* Distribution and Censor Counts tables											
	***************************************************************************************************/
	options orientation=landscape;
	
	/* Distribution tables where each column is a category */
	%macro categorytables(data=, title=, totalheader=);
		&report_header.;
		ods proclabel = "&title.";
		options missing=' ';
		proc report data=&data. nofs nowd contents=''
			style(header)=[rules=none background=ligr font_weight=bold font_size=8pt fontfamily=arial vjust=b bordertopcolor=black borderbottomcolor=black] split='*'
			style(report)=[rules=none frame=box fontfamily=arial cellpadding =&line_spacing.pt];

			column ("^S={just=l} &title." order stratsort stratsort2 groupn
                     ("^S={background=ligr borderleftcolor=ligr} &totalheader" total_count total_percent) 
					  %do s = 1 %to %eval(&num_categories);
						 %let display = %scan(&categories., &s, %str( ));
						 ("^S={background=ligr borderleftcolor=ligr}&display" _&s. _&s._percent)
                      %end;
					);

			    define order /order noprint order=data;
			  	define stratsort / order noprint order=data;
			  	define stratsort2 / order noprint order=data;
				define groupn /display 'Exposures'
			    	style(column)=[font_size=8pt fontfamily=arial just=l width=2.1in] style(header)=[just=C background=white borderbottomcolor=black];
			    define total_count / display 'N'  
				    style(column)=[font_size=8pt fontfamily=arial just=C width=55pt] style(header)=[just=C background=white borderbottomcolor=black];
			    define total_percent / display '%'  
				    style(column)=[font_size=8pt fontfamily=arial just=C width=25pt] style(header)=[just=C background=white borderbottomcolor=black];

			%do s=1 %to %eval(&num_categories);
			    define _&s. / display 'N'  
				    style(column)=[font_size=8pt fontfamily=arial just=C width=55pt] style(header)=[just=C background=white borderbottomcolor=black];
			    define _&s._percent / display '%'  
				    style(column)=[font_size=8pt fontfamily=arial just=C width=25pt] style(header)=[just=C background=white borderbottomcolor=black];
			%end;

			/*Add indentation*/
		  	compute groupn;
				if stratsort ne 0 and stratsort2 = 0 then call define(_col_,'style','style={leftmargin=.1in}');
		    	if stratsort ne 0 and stratsort2 ne 0 then call define(_col_,'style','style={leftmargin=.2in}');
		    endcomp;

			/*Add shading to overall group*/
			%if "&tablesub." ne "A" %then %do;
		  	compute stratsort;
				if stratsort =0 then call define(_row_,'style','style={font_weight=bold}');
			endcomp;
			%end;
		run;
		options missing='.';

	%mend;

	/* Continuous distribution metrics */
	%macro continuoustables(data=, title=, totalheader=);
	    &report_header.;
		ods proclabel = "&title.";

			%if "&table." ne "T13" %then %do;
				%let nwidth = 100;
				%let cwidth = 65;
			%end;
			%if "&table." = "T13" %then %do;
				%let nwidth = 82;
				%let cwidth = 47;
			%end;

		proc report data=&data. nofs nowd contents=''
			style(header)=[rules=none background=ligr font_weight=bold font_size=8pt fontfamily=arial vjust=b bordertopcolor=black borderbottomcolor=black] split='*'
			style(report)=[rules=none frame=box fontfamily=arial cellpadding =&line_spacing.pt];

			column ("^S={just=l}&title." (order stratsort stratsort2 groupn 
							%if "&table." = "T13" %then %do;
							gaplabel
							%end;
			         (n mean std min p25 median p75 max)));

			   	define order / order noprint order=data;
			   	define stratsort / display noprint order=data;
			   	define stratsort2 / display noprint order=data;
			   	define groupn /display 'Exposures'
			       style(column)=[font_size=8pt fontfamily=arial width=2.1in just=l] style(header)=[just=C background=white borderbottomcolor=black];
				%if "&table." = "T13" %then %do;
			 	define gaplabel /display 'Gap'
			       style(column)=[font_size=8pt fontfamily=arial width=1.8in just=l] style(header)=[just=C background=white borderbottomcolor=black];
				%end;
				define n / display "&totalheader."  
				   style(column)=[font_size=8pt fontfamily=arial just=c width=&nwidth.pt] style(header)=[just=C background=white borderbottomcolor=black];
				define mean / display 'Mean'  
						style(column)=[font_size=8pt fontfamily=arial just=c width=&cwidth.pt] style(header)=[just=C background=white borderbottomcolor=black];
				define std / display 'STD'  
						style(column)=[font_size=8pt fontfamily=arial just=c width=&cwidth.pt] style(header)=[just=C background=white borderbottomcolor=black];
				define min / display 'Min'  
						style(column)=[font_size=8pt fontfamily=arial just=c width=&cwidth.pt] style(header)=[just=C background=white borderbottomcolor=black];
				define p25 / display 'Q1'  
						style(column)=[font_size=8pt fontfamily=arial just=c width=&cwidth.pt] style(header)=[just=C background=white borderbottomcolor=black];
				define median / display 'Median'  
						style(column)=[font_size=8pt fontfamily=arial just=c width=&cwidth.pt] style(header)=[just=C background=white borderbottomcolor=black];
				define p75 / display 'Q3'  
						style(column)=[font_size=8pt fontfamily=arial just=c width=&cwidth.pt] style(header)=[just=C background=white borderbottomcolor=black];
				define max / display 'Max'  
						style(column)=[font_size=8pt fontfamily=arial just=c width=&cwidth.pt] style(header)=[just=C background=white borderbottomcolor=black];

		 	/*Add indentation*/
		  	compute groupn;
				if stratsort ne 0 and stratsort2 = 0 then call define(_col_,'style','style={leftmargin=.1in}');
		    	if stratsort ne 0 and stratsort2 ne 0 then call define(_col_,'style','style={leftmargin=.2in}');
		    endcomp;

			/*Add shading to overall group*/
			%if "&tablesub." ne "A" %then %do;
		  	compute stratsort;
				if stratsort =0 then call define(_row_,'style','style={font_weight=bold}');
			endcomp;
			%end;
		run;

	%mend;

	/***************************/
	/* loop through all tables */
	/***************************/

	%isdata(dataset=_outputtables);
	%let numtables = &nobs.;

	%if %eval(&nobs.>0) %then %do;

		%do t = 1 %to &numtables.;

			data _null_;
				set _outputtables;
					if _n_=&t. then do;
						call symputx('table', table);
						call symputx('tablesub', tablesub);
						call symputx('disptablenum', disptablenum);
					end;
			run;

			*reset macro vars;
			%let num_categories = 0;
			%let categories = ;

			/*Assign stratification to title*/
			/*overall*/
			%if "&tablesub" = "A" %then %let stratatitle = ;
			/*sex*/
			%if "&tablesub" = "B" %then %let stratatitle = , by sex ;
			/*age group*/
			%if "&tablesub" = "C" %then %let stratatitle = , by age group ;
			/*race*/
			%if "&tablesub" = "D" %then %let stratatitle = , by race ;
			/*hispanic*/
			%if "&tablesub" = "E" %then %let stratatitle = , by hispanic ethnicity;
			/*sex * age group*/
			%if "&tablesub" = "BC" %then %let stratatitle = , by sex, age group ;
			/*sex * race*/
			%if "&tablesub" = "BD" %then %let stratatitle = , by sex, race ;
			/*sex * hispanic*/
			%if "&tablesub" = "BE" %then %let stratatitle = , by sex, hispanic ethnicity ;
			/*age group * race*/
			%if "&tablesub" = "CD" %then %let stratatitle = , by age group, race ;
			/*age group * hispanic*/
			%if "&tablesub" = "CE" %then %let stratatitle = , by age group, hispanic ethnicity ;

			%if "&table." = "T2" %then %do;
				%let categories = &totepisdurstrat.;
				%let num_categories = %sysfunc(countw(&categories, ' '));

				%categorytables(data=table_&table.&tablesub.,
								title=%bquote(Table &disptablenum.: Distribution of cumulative exposure duration, by length categories, in days&stratatitle),
								totalheader=Total Patients);
			%end;
			%if "&table." = "T3" %then %do;
				%continuoustables(data=table_&table.&tablesub.,
								  title=%bquote(Table &disptablenum.: Descriptive statistics of cumulative exposure duration, all episodes, in days&stratatitle),
								  totalheader=Total Patients);
			%end;
			%if "&table." = "T4" %then %do;
				%let categories = &episdurstrat.;
				%let num_categories = %sysfunc(countw(&categories, ' '));

				%categorytables(data=table_&table.&tablesub.,
								title=%bquote(Table &disptablenum.: Distribution of first exposure episode duration, by length categories, in days&stratatitle),
								totalheader=Total Episodes);
			%end;
			%if "&table." = "T5" %then %do;
				%continuoustables(data=table_&table.&tablesub.,
								  title=%bquote(Table &disptablenum.: Descriptive statistics of first exposure episode duration, in days&stratatitle),
								  totalheader=Total Episodes);
			%end;
			%if "&table." = "T6" %then %do;
				%let categories = &episdurstrat.;
				%let num_categories = %sysfunc(countw(&categories, ' '));

				%categorytables(data=table_&table.&tablesub.,
								title=%bquote(Table &disptablenum.: Distribution of second and subsequent exposure episode duration, by length categories, in days&stratatitle),
								totalheader=Total Episodes);
			%end;
			%if "&table." = "T7" %then %do;
				%continuoustables(data=table_&table.&tablesub.,
								  title=%bquote(Table &disptablenum.: Descriptive statistics of second and subsequent exposure episode duration, in days&stratatitle),
								  totalheader=Total Episodes);
			%end;
			%if "&table." = "T8" %then %do;
				%let categories = &episdurstrat.;
				%let num_categories = %sysfunc(countw(&categories, ' '));

				%categorytables(data=table_&table.&tablesub.,
								title=%bquote(Table &disptablenum.: Distribution of all episode durations, by length categories, in days&stratatitle),
								totalheader=Total Episodes);
			%end;
			%if "&table." = "T9" %then %do;
				%continuoustables(data=table_&table.&tablesub.,
								  title=%bquote(Table &disptablenum.: Descriptive statistics of all exposure episode durations, in days&stratatitle),
								  totalheader=Total Episodes);
			%end;
			%if "&table." = "T10" %then %do;
				%let categories = &dispdayssupstrat.;
				%let num_categories = %sysfunc(countw(&categories, ' '));

				%categorytables(data=table_&table.&tablesub.,
								title=%bquote(Table &disptablenum.: Distribution of days supplied per dispensing, by length categories&stratatitle.),
								totalheader=Total Dispensings);
			%end;
			%if "&table." = "T11" %then %do;
				%continuoustables(data=table_&table.&tablesub.,
								  title=%bquote(Table &disptablenum.: Descriptive statistics of days supplied per dispensing&stratatitle.),
								  totalheader=Total Dispensings);
			%end;
			%if "&table." = "T12" %then %do;
				%continuoustables(data=table_&table.&tablesub.,
								  title=%bquote(Table &disptablenum.: Descriptive statistics of the length of the first gap between treatment episodes, in days&stratatitle.),
								  totalheader=Total Gaps);
			%end;
			%if "&table." = "T13" %then %do;
				%continuoustables(data=table_&table.&tablesub.,
								  title=%bquote(Table &disptablenum.: Descriptive statistics of the length of second and subequent gaps between treatment episodes, in days&stratatitle.),
								  totalheader=Total Gaps);
			%end;
			%if "&table." = "T14" %then %do;
				%continuoustables(data=table_&table.&tablesub.,
								  title=%bquote(Table &disptablenum.: Descriptive statistics of the length of all gaps between treatment episodes, in days&stratatitle.),
								  totalheader=Total Gaps);
			%end;
			%if "&table." = "T15" %then %do;

	   			ods proclabel = "Table &disptablenum. Counts of reason for censoring, all episodes and first episode";
	   			&report_header.;

				proc report data=censor_table nofs nowd contents=''
					style(header)=[rules=none background=ligr font_weight=bold font_size=8pt fontfamily=arial vjust=b bordertopcolor=black borderbottomcolor=black] split='*'
				  	style(report)=[rules=none frame=box fontfamily=arial cellpadding =&line_spacing.pt];

				  	column ("^S={just=l}Table &disptablenum.: Counts of reason for censoring, all episodes and first episode" (descr exposure order 
						   ("^S={borderleftcolor=black borderrightcolor=black}Total" count percent_total) 
						   ("^S={borderrightcolor=black}Disenrollment" censn_elig percent_elig)
						   %if "&displaydth" = "Y" %then %do;
						   ("^S={borderrightcolor=black}Evidence of death" censn_dth percent_dth)
						   %end;
						   %if "&displaydpqry" = "Y" %then %do;
  						   ("^S={borderrightcolor=black}DP/Query end date" censn_dpqry percent_dpqry) 
						   %end;
						   ("^S={borderrightcolor=black}Episode end" censn_episend percent_episend)
						   %if "&displayspec" = "Y" %then %do;
						   ("^S={borderrightcolor=black}Additional criteria" censn_spec percent_spec)
						   %end;
							));
				    define descr / order noprint order=data;
					define order/ order noprint order=data;
					define exposure /display ''
					    style(column)=[font_size=8pt fontfamily=arial just=l borderrightcolor=black  width=2.1in] style(header)=[just=C background=white borderbottomcolor=black borderrightcolor=black ];
					define count / display 'N'  
					   style(column)=[font_size=8pt fontfamily=arial just=c  width=62pt] style(header)=[just=C background=white borderbottomcolor=black];
				    define percent_total / display '%'  
					   style(column)=[font_size=8pt fontfamily=arial just=c  borderrightcolor=black  width=25pt] style(header)=[just=C background=white borderbottomcolor=black borderrightcolor=black];
					define censn_elig / display 'N'  
					   style(column)=[font_size=8pt fontfamily=arial just=c  width=62pt] style(header)=[just=C background=white borderbottomcolor=black];
				    define percent_elig / display '%'  
					   style(column)=[font_size=8pt fontfamily=arial just=c  borderrightcolor=black  width=25pt] style(header)=[just=C background=white borderbottomcolor=black borderrightcolor=black];
					%if "&displaydth" = "Y" %then %do;
					define censn_dth / display 'N'  
					   style(column)=[font_size=8pt fontfamily=arial just=c  width=62pt] style(header)=[just=C background=white borderbottomcolor=black];
				    define percent_dth / display '%'  
					   style(column)=[font_size=8pt fontfamily=arial just=c  borderrightcolor=black  width=25pt] style(header)=[just=C background=white borderbottomcolor=black borderrightcolor=black];
					%end;
					%if "&displaydpqry" = "Y" %then %do;
					define censn_dpqry / display 'N'  
					   style(column)=[font_size=8pt fontfamily=arial just=c  width=62pt] style(header)=[just=C background=white borderbottomcolor=black];
				    define percent_dpqry / display '%'  
					   style(column)=[font_size=8pt fontfamily=arial just=c  borderrightcolor=black  width=25pt] style(header)=[just=C background=white borderbottomcolor=black borderrightcolor=black];
					%end;
					define censn_episend / display 'N'  
					   style(column)=[font_size=8pt fontfamily=arial just=c  width=62pt] style(header)=[just=C background=white borderbottomcolor=black];
				    define percent_episend / display '%'  
					   style(column)=[font_size=8pt fontfamily=arial just=c  borderrightcolor=black  width=25pt] style(header)=[just=C background=white borderbottomcolor=black borderrightcolor=black];
					%if "&displayspec" = "Y" %then %do;
					define censn_spec / display 'N'  
					   style(column)=[font_size=8pt fontfamily=arial just=c  width=62pt] style(header)=[just=C background=white borderbottomcolor=black];
				    define percent_spec / display '%'  
					   style(column)=[font_size=8pt fontfamily=arial just=c  borderrightcolor=black  width=25pt] style(header)=[just=C background=white borderbottomcolor=black borderrightcolor=black];
					%end;

				    *Add spanning header;
					compute before descr / style=[background=ligr fontfamily=arial font_size=8pt just=L font_weight=bold bordertopcolor=black borderbottomcolor=black];
						length text $100;
						if descr = "Episodes" then do; 
							text='Exposures'; 
							num=100;
						end;
						else if descr = "Patients" then do; 
							text="Patients' First Episode"; 
							num=100; 
						end;
						else do; 
							text = "";
							num=0;
						end;
						line text $Varying. num; 
					endcomp;
			   run;
			%end;

			%if "&table." = "T16" %then %do;

			*** Attrition Table ***;	
			ods proclabel = "Table &disptablenum.: Cohort Attrition Table";
			&report_header.;
		      	  
			proc report data = attrition nofs nowd headline headskip split="*" contents=''
					style(header)=[rules=none background=white font_weight=bold font_size=8pt fontfamily=arial vjust=b bordertopcolor=black borderbottomcolor=black] split='*'
					style(report)=[rules=none frame=box fontfamily=arial cellpadding =&line_spacing.pt];
					
					columns ( "^S={just=l background=ligr borderrightcolor=ligr} Table &lasttablenum.: Cohort Attrition Table" order  exposure descr percent);			
					define order/group noprint;
					define exposure/group noprint;
					define descr / order=data 'Attrition Criteria' style(column)=[rules=none font_size=8pt fontfamily=arial width=620pt just=L];
					define percent / order=data  'Members' style(column)=[font_size=8pt width=110pt fontfamily=arial just=L] ;

					/*Add header line*/
					compute before order / style=[background=ligr fontfamily=arial font_size=8pt just=L font_weight=bold bordertopcolor=black borderbottomcolor=black];
						length text $100;
						%do grps=1 %to &num_allgroups.;
						  if order =&grps. then do; 
						    text="&&exposure&grps"; 
							num=100;
						  end;
						%end;
						line text $Varying. num;
					 endcomp;
				run;

			%end;
		%end;
	%end;

	/**************************************************************************************************
	* Figures										
	***************************************************************************************************/
	options orientation = portrait;;
	ods graphics on /height=4.2in width=5in scale=on;
	ods startpage=no;

	/*Macros*/
	%macro figure123_overall_output(title=, yvar=, yvar_label=);
	  	proc odstext pagebreak=yes; 
			p "^S={fontfamily=arial font_size=8pt font_weight=bold  just=L}&title.";
			p " ";
		run;
	
		ods proclabel = "&title.";
		proc sgplot data = figure123_&order._a noborder noautolegend;
			series x= mntsfromstart y=&yvar. / name='raw' lineattrs=(pattern=solid color=darkred thickness=2 );
			series x= mntsfromstart y=cumulative_&yvar. / name= 'cumul' lineattrs=(pattern=shortdash color=darkred thickness=2 );

			xaxis label = "Months after Study Start" labelattrs=(size=7 color=black) valueattrs=(size=7 color=gray) min=1;
			yaxis label = "&yvar_label" labelattrs=(size=7 color=black) valueattrs=(size=7 color=gray);
		run;
		ods text = "^S={fontfamily=arial font_size=6.5pt just=L}Note: Cumulative users represented by dashed line";
	%mend;

	%macro figure123_stratified_output(stratname=, title=, yvar=, yvar_label=);		
		proc odstext pagebreak=yes;
			p "^S={fontfamily=arial font_size=8pt font_weight=bold  just=L}&title.";
			p " ";
		run;

		ods proclabel = "&title.";
		proc sgplot data = figure123_&order._&tablesub. noborder dattrmap = attrmap_&order.;
			series x= mntsfromstart y=&yvar. / group=label name='raw' attrid=&stratname. lineattrs=(pattern=solid thickness=2 );
			series x= mntsfromstart y=cumulative_&yvar. / group=label name= 'cumul' attrid=&stratname. lineattrs=(pattern=shortdash thickness=2);

			xaxis label = "Months after Study Start" labelattrs=(size=7 color=black) valueattrs=(size=7 color=gray) min=1;
			yaxis label = "&yvar_label" labelattrs=(size=7 color=black) valueattrs=(size=7 color=gray);
			keylegend 'raw' / valueattrs=(size=7 color=black) location=outside position=bottom noborder linelength=.25in;
		run;
		ods text = "^S={fontfamily=arial font_size=6.5pt just=L}Note: Cumulative users represented by dashed line" ;
	%mend;

	%macro figure4();
		proc odstext %if %sysfunc(mod(&order.,2))=0 %then %do; pagebreak=yes %end; ;
			p "^S={fontfamily=arial font_size=8pt font_weight=bold just=L}Figure &disptablenum..&order.. Survival curve for length of first treatment episode (&exposure.)";
			p " ";
		run;
	
		ods proclabel = "Figure &disptablenum..&order. Survival curve for length of first treatment episode";
		
		proc sgplot data = figure4_&order. noborder;
			/*Output 1 step statement per censor criteria*/
			%do cnt = 1 %to &&censor_cnt&order.;
			step x= episodelength y=%sysfunc(cats(km_,%quote(%scan(&&censor_list&order., &cnt.))))/ lineattrs=(thickness=2 pattern=solid color=%scan(&colors., &cnt.));
			%end;
			xaxis label = "Days" labelattrs=(size=7 color=black) valueattrs=(size=7 color=gray)
				 %if %isBlank(&km_f4_xmin.) = 0 %then %do; values = (&km_f4_xmin. to &km_f4_xmax. by &km_f4_xtick.); %end; ;
			yaxis label = "Survival Probability" labelattrs=(size=7 color=black) valueattrs=(size=7 color=gray) values = (&km_f4_ymin. to &km_f4_ymax. by &km_f4_ytick.);
			keylegend  / valueattrs=(size=7 color=black) location=outside position=bottom noborder linelength=.25in;
		run;

		*insert an empty line so that the graph will come out in the right page;
		ods text = "          ";
	%mend figure4;

	%macro figure5();
		proc odstext %if %sysfunc(mod(&order.,2))=0 %then %do; pagebreak=yes %end; ;
			p "^S={fontfamily=arial font_size=8pt font_weight=bold just=L}Figure &disptablenum..&order.. Survival curve for length of first treatment episode (&figure5label.)";
			p " ";
		run;
	
		ods proclabel = "Figure &disptablenum..&order. Survival curve for length of first treatment episode (&figure5label.)";
		
		proc sgplot data = figure5 noborder;
			/*Output 1 step statement per censor criteria*/
			%do cnt = 1 %to &figure5count.;
			step x= episodelength y=%sysfunc(cats(km_&cnt._&censoring_display_F5.))/ lineattrs=(thickness=2 pattern=solid color=%scan(&colors., &cnt.));
			%end;
			xaxis label = "Days" labelattrs=(size=7 color=black) valueattrs=(size=7 color=gray)
				 %if %isBlank(&km_f5_xmin.) = 0 %then %do; values = (&km_f5_xmin. to &km_f5_xmax. by &km_f5_xtick.); %end; ;
			yaxis label = "Survival Probability" labelattrs=(size=7 color=black) valueattrs=(size=7 color=gray) values = (&km_f5_ymin. to &km_f5_ymax. by &km_f5_ytick.);
			keylegend  / valueattrs=(size=7 color=black) location=outside position=bottom noborder linelength=.25in;
		run;

		*insert an empty line so that the graph will come out in the right page;
		ods text = "          ";
	%mend figure5;

	/************************/
	/* loop through figures */
	/************************/

	%isdata(dataset=_outputfigures);
	%let numfigures = &nobs.;

	%if %eval(&nobs.>0) %then %do;

		%do f = 1 %to &numfigures.;

			%if %eval(&f. ne 1) %then %do;
			    ods startpage = now;
			%end;

			data _null_;
				set _outputfigures;
					if _n_=&f. then do;
						call symputx('table', table);
						call symputx('tablesub', tablesub);
						call symputx('disptablenum', disptablenum);
					end;
			run;

			%if "&tablesub." = "B" %then %do;
				%let strata = , by sex;
				%let sub = sex;
			%end;
			%if "&tablesub." = "C" %then %do;
				%let strata = , by age group;
				%let sub = agegroup;
			%end;
			%if "&tablesub." = "D" %then %do;
				%let strata = , by race;
				%let sub = race;
			%end;
			%if "&tablesub." = "E" %then %do;
				%let strata = , by hispanic ethnicity;
				%let sub = hispanic;
			%end;

			/* Figures 1, 2, and 3 */
			%if %sysfunc(prxmatch(m/F1|F2|F3/i,&table..)) > 0 %then %do;

				%if "&table"="F1" %then %do;
		     		%let tmpyvar=npts;
		     		%let tmpyvar_label=Number of New Users;
		     		%let tmptitl=New users per month post-study start + cumulative new users per month since study start;
		   		%end;
		   		%if "&table"="F2" %then %do;
		    		%let tmpyvar=adjustedcodecount;
		     		%let tmpyvar_label=Number of Dispensings;
		     		%let tmptitl=Dispensings per month post-study start + cumulative new users per month since study start;
		  		%end;
		   		%if "&table"="F3" %then %do;
		     		%let tmpyvar=daysupp;
		     		%let tmpyvar_label=Total Days supply;
		     		%let tmptitl=Days supply per month post-study start + cumulative new users per month since study start;
		   		%end;

				&report_header.;

		      	%do order=1 %to &num_allgroups.;
					
					data _null_;
						set &groupfile.(where=(order=&order.));
						if _n_ = 1 then do;
							call symputx('exposure', exposure);
						end;
					run;

					%if "&tablesub"="A" %then %do;
					   ods proclabel = "Figure &disptablenum..&order. (&exposure.)";
				       %figure123_overall_output(title=%bquote(Figure &disptablenum..&order.. &tmptitl. (&exposure.)),
												 yvar=&tmpyvar.,
												 yvar_label=&tmpyvar_label.);
					%end;
					%else %do;
					    ods proclabel = "Figure &disptablenum..&order. (&exposure.)";
					    %figure123_stratified_output(stratname=&sub.,
													 title=%bquote(Figure &disptablenum..&order.. &tmptitl.&strata. (&exposure.)),
											         yvar = &tmpyvar.,
											         yvar_label =&tmpyvar_label.);
					%end;

				%end; /*loop through groups*/
		 	%end; /*figures 1, 2, 3*/

			/* Figure 4 */
			%if %sysfunc(prxmatch(m/F4/i,&table..)) > 0 %then %do;

				&report_header.;
		      	%do order=1 %to &num_allgroups.;

					ods proclabel = "Figure &disptablenum..&order.";

					data _null_;
					   set &groupfile.(where=(order=&order.));
					   call symputx("exposure", exposure);
					run;

					/*call macro*/
					%figure4();
	       
				%end; /*loop through groups*/
			%end; /*Figure 4*/

			/* Figure 5*/
			%if %sysfunc(prxmatch(m/F5/i,&table..)) > 0 %then %do;

				&report_header.;
				ods proclabel = "Figure &disptablenum..&order.";
				/*call macro*/
				%figure5();

			%end; /*figure 5*/

		%end; /*loop through figures*/
	%end; /*output figures*/


	/**************************************************************************************************
	* Start and End dates for each DP											
	***************************************************************************************************/
	ods startpage = now;

	&report_header.;

	ods proclabel = "Dates of Available Data for Each Data Partner (DP) up to Request End Date (&enddate.) as of Query Distribution Date";

	 proc report data=dpdates nofs nowd 	
		style(header)=[rules=none background=darkgrey font_weight=bold font_size=8pt fontfamily=arial vjust=b bordertopcolor=black borderbottomcolor=black] split='*'
		style(report)=[rules=none fontfamily=arial frame=box];

		column ("^S={just=L background=white}Dates of Available Data for Each Data Partner (DP) up to Request End Date (&enddate.) as of Query Distribution Date"
				maskedID dpmin maxdate); 

		define maskedID / display 'Data Partner (Masked)' 
			style(column)=[font_size=8pt fontfamily=arial width=3in just=C]  style(header)=[just=C background=darkgrey borderbottomcolor=black font_weight = bold];
		define dpmin / display 'Start Date' 
			style(column)=[font_size=8pt fontfamily=arial width=1.75in just=C]  style(header)=[just=C background=darkgrey borderbottomcolor=black font_weight = bold];
		define maxdate / display 'End Date' 
			style(column)=[font_size=8pt fontfamily=arial width=1.75in just=C] style(header)=[just=C background=darkgrey borderbottomcolor=black font_weight = bold]; 
	 run;

	ods _all_ close;
	ods results;
	ods listing;

    %put NOTE: ******** END OF MACRO: ms_t5_output_report v2.1 ********;

%mend ms_t5_output_report;
